#include <avr/io.h>
#include <util/delay.h>

// B0 | ADC0 | AIN0 | OCRA ||| ADC3 | B3
//                     GND ||| VCC
// B1 | ADC1 | AIN1 | OCRB ||| ADC2 | B2

// Oversampling:
// B0 <- input with 2k2 impedance and 1uF cap
// B2 <- uart out

extern void asm_write_char(uint8_t c);

int main (void)
{
    // Tune oscillator to 8 MHz
    CCP = 0xD8;
    CLKPSR = 0;
    OSCCAL = 159;
    
    // Uart out
    DDRB = (1<<PB2);
    PORTB = (1<<PB2);
    
    // Input nalog
    DIDR0 = (1<<0);
    
    // Setup ADC
    // 125kHz -> 112us per sample
    ADMUX = 0;
    ADCSRA = (1<<ADEN) | 6;
    
    while(1) {
        uint16_t value = 0;
        
        /*
        // No oversampling
        for (uint8_t i = 0; i < 64; ++i) {
            ADCSRA |= 1<<ADSC;
            while(ADCSRA & (1<<ADSC));
            value += ADCL;
        }
        */
        
        /*
        // Pull-up
        for (uint8_t i = 0; i < 32; ++i) {
            ADCSRA |= 1<<ADSC;
            while(ADCSRA & (1<<ADSC));
            value += ADCL;
        }
        PUEB |= 1<<0;
        for (uint8_t i = 0; i < 32; ++i) {
            ADCSRA |= 1<<ADSC;
            while(ADCSRA & (1<<ADSC));
            value += ADCL;
        }
        PUEB &= ~(1<<0);
        */
        
        // Oversampling dischage only
        DDRB |= 1;
        _delay_us(100); // discharge cap
        DDRB &= ~1;
        for (uint8_t i = 0; i < 64; ++i) {
            ADCSRA |= 1<<ADSC;
            while(ADCSRA & (1<<ADSC));
            value += ADCL;
        }
        
        /*
        // Oversampling charge/dischage
        DDRB |= 1;
        _delay_us(100); // discharge cap
        DDRB &= ~1;
        for (uint8_t i = 0; i < 32; ++i) {
            ADCSRA |= 1<<ADSC;
            while(ADCSRA & (1<<ADSC));
            value += ADCL;
        }
        _delay_ms(20);
        PORTB |= 1;
        DDRB |= 1;
        _delay_us(100); // charge cap
        DDRB &= ~1;
        PORTB &= ~1;
        for (uint8_t i = 0; i < 32; ++i) {
            ADCSRA |= 1<<ADSC;
            while(ADCSRA & (1<<ADSC));
            value += ADCL;
        }
        */
        
        // Send value
        asm_write_char(value>>8);
        asm_write_char(value);
        _delay_ms(50);
    }

    return (0);
}

